@@ -1,5 +1,5 @@
 # $File: //depot/cpanplus/dist/lib/CPANPLUS/Internals/Install.pm $
-# $Revision: #3 $ $Change: 3456 $ $DateTime: 2003/01/12 12:16:32 $
+# $Revision: #7 $ $Change: 7683 $ $DateTime: 2003/08/23 18:20:12 $
 
 #######################################################
 ###           CPANPLUS/Internals/Install.pm         ###
@@ -12,11 +12,13 @@
 package CPANPLUS::Internals::Install;
 
 use strict;
+use Cwd;
 use DirHandle;
 use FileHandle;
 use Data::Dumper;
 use CPANPLUS::I18N;
-
+use CPANPLUS::Tools::Check qw[check];
+use CPANPLUS::Tools::Module qw[check_install];
 BEGIN {
     use vars        qw( $VERSION );
     $VERSION    =   $CPANPLUS::Internals::VERSION;
@@ -25,15 +27,23 @@ BEGIN {
 ### method to install modules from CPAN. does everything from fetching the file, to installing it ###
 sub _install_module {
     my $self = shift;
-    my %args = @_;
+    my %hash = @_;
+    my $conf = $self->configure_object;
+    my $err  = $self->error_object;
 
-    my $conf = $self->{_conf};
-    my $err  = $self->{_error};
+    local $CPANPLUS::Tools::Check::ALLOW_UNKNOWN = 1;
 
-    ### list of modules we've installed in this session ###
-    my $mods;
+    my $tmpl = {
+        modules => { required => 1, default => [], strict_type => 1 },
+        verbose => { default => $conf->get_conf('verbose') },
+    };
 
-    for my $href ( @{$args{'modules'}} ) {
+    my $args = check( $tmpl, \%hash ) or return undef;
+
+    my $verbose     = $args->{verbose};
+    my $mods        = {};
+
+    for my $modobj ( @{ delete $args->{'modules'} } ) {
 
         ### flag to see if an install failed somewhere ###
         my $fail = 0;
@@ -47,10 +57,10 @@ sub _install_module {
             ### should work well enough; -kane
             ### let's allow subclassing here -autrijus
 
-            unless ( UNIVERSAL::isa( $href, 'CPANPLUS::Internals::Module' ) ) {
+            unless ( UNIVERSAL::isa( $modobj, 'CPANPLUS::Internals::Module' ) ) {
                 $err->trap(
-                    error => loc("You did not pass a proper module object! %1 (%2)", "@_", $href).
-                             ref($href),
+                    error => loc("You did not pass a proper module object! %1 (%2)", "@_", $modobj).
+                             ref($modobj),
                 );
 
                 ### mark we did nothing for this module ###
@@ -59,14 +69,14 @@ sub _install_module {
             }
 
             ### we use the name way too often, time to shortcut... ###
-            $m = $href->module();
+            $m = $modobj->module();
 
             ### store we actually did something here ###
-            $mods->{$m} = $href;
+            $mods->{$m} = $modobj;
 
             ### check if any of the prereqs we're about to install wants us to get
             ### a newer version of perl... if so, skip, we dont want to upgrade perl
-            if ($m =~ /^base$/i or $href->{package} =~ /^perl-\d/i ) {
+            if ($m =~ /^base$/i or $modobj->package =~ /^perl-\d/i ) {
                 $err->inform(
                     msg => loc("Cannot fetch %1 (part of core perl distribution); skipping", $m),
                 );
@@ -79,8 +89,8 @@ sub _install_module {
             my $is_bundle = 1 if $m =~ /^bundle::/i;
 
             ### get a hashref with all module information ###
-            #my $href;
-            #unless( $href = $self->_module_tree->{$m} ) {
+            #my $modobj;
+            #unless( $modobj = $self->_module_tree->{$m} ) {
             #    $err->trap( error => "could not find information for module $m" );
             #    $flag = 1;
             #    next;
@@ -89,7 +99,7 @@ sub _install_module {
             ### pass it to fetch, it knows how to get the right dir
             ### get the name of the file it fetched back
             my $file;
-            unless ( $file = $self->_fetch(data => $href) ) {
+            unless ( $file = $self->_fetch( data => $modobj ) ) {
                 $err->trap( error => qq[Could not fetch module $m] );
 
                 $fail = 1;
@@ -97,7 +107,7 @@ sub _install_module {
             }
 
             ### store where we fetched the file ###
-            $href->{status}->{fetch} = $file;
+            $modobj->status->fetch( $file );
 
             ### in case of MD5 checks, we need the checksums and do the MD5
             ### check on them.
@@ -107,10 +117,7 @@ sub _install_module {
 
 
             if ($check_md5) {
-                unless ( $self->_check_md5(
-                            data        => $href,
-                        )
-                 ) {
+                unless ( $self->_check_md5( data => $modobj ) ) {
                     $err->trap( error => loc("Checksums did not match for %1", $file) );
 
                     $fail = 1;
@@ -118,25 +125,31 @@ sub _install_module {
                 }
             }
 
-            $href->{status}->{md5} = 1;
+            $modobj->status->md5(1);
+
+            my $check_signature = $conf->get_conf( 'signature' );
+            if ($check_md5 and $check_signature) {
+                # checking for detached signature, if any.
+                unless ($self->_check_detached_signature( mod => $modobj )) {
+                    $err->trap( error => loc("Signature checking failed for %1", $file) );
+                    $fail = 1;
+                    last LOOP;
+                }
+            }
 
             ### extract the file, get the location it extracted to
             my $dir;
-            unless ( $dir = $self->_extract( data => $href, %args ) ) {
+            unless ( $dir = $self->_extract( data => $modobj, %$args ) ) {
                 $err->trap( error => loc("Could not obtain extraction directory for module %1", $m) );
 
                 $fail = 1;
                 last LOOP;
             }
 
-            $href->{status}->{extract} = $dir;
+            $modobj->status->extract( $dir );
 
-            my $check_signature = $conf->get_conf( 'signature' );
             if ($check_signature) {
-                unless ( $self->_check_signature(
-                            dir        => $dir,
-                        )
-                 ) {
+                unless ( $self->_check_signature( dir => $dir ) ) {
                     $err->trap( error => loc("Signature checking failed for %1", $file) );
 
                     $fail = 1;
@@ -144,15 +157,16 @@ sub _install_module {
                 }
             }
 
-            $href->{status}->{signature} = 1;
+            $modobj->status->signature(1);
 
             ### make'ing the target. we get a return value
             ### that is a hash ref of module names and their objects
             ### we'll have to examine the rv to see if the overall
             ### make for *this* module went ok
-            my $rv = $self->_make(dir => $dir, module => $href, %args);
+            my $rv = $self->_make(dir => $dir, module => $modobj, %$args);
 
-            unless ( $rv->{ $href->module() }->{make}->{overall} ) {
+            ### XXX this needs fixink for new Status.pm!!! XXX ###
+            unless ( $rv->{ $modobj->module() }->{make}->{overall} ) {
                 $err->trap( error => loc("An error occurred handling module %1", $m) );
 
                 $fail = 1;
@@ -160,12 +174,13 @@ sub _install_module {
             };
 
             ### store the status explicitly for the module of this session ###
-            $href->{status}->{make} = $rv->{$m}->{make};
+            ### XXX this needs fixink for new Status.pm!!! XXX ###
+            #$modobj->status->make( $rv->{$m}->{make} );
 
             ### make's rv is special, since it can contain full module install's
             ### data, and also for modules then we tried to install here (prereqs)
             ### we'll have to loop over the keys and store that we fiddled with them
-            for my $modname ( keys %$rv ) {
+            for my $modname ( sort keys %$rv ) {
                 ### obsolete, we'll overwrite anyway at the end of the loop
                 #unless( $m eq $modname ) {
                     $mods->{$modname} = $self->module_tree->{$modname}
@@ -174,21 +189,23 @@ sub _install_module {
 
             if ( $is_bundle ) {
                 my $res;
-                unless ( $res = $self->_install_bundle( dir => [ $dir ], %args) ) {
+                unless ( $res = $self->_install_bundle( dir => [ $dir ], %$args) ) {
                     $err->trap( error => loc("An error occurred handling bundle %1", $m) );
 
                     $fail = 1;
                     last LOOP;
                 };
-                $href->{status}->{bundle} = $res;
+                $modobj->status->bundle( $res );
             }
 
         } # LOOP
 
-        $href->{status}->{install} = !$fail;
+        # don't set this, since we may have not actually installed,
+        # but just tested or so
+        #$modobj->status->install( !$fail );
 
         ### refresh the data in $mods ###
-        $mods->{$m} = $href;
+        $mods->{$m} = $modobj;
     }
 
 
@@ -196,7 +213,11 @@ sub _install_module {
     ### and base our return value on that
     ### return $flag ? 0 : 1;
 
-    my %return = map { my $m = $_->module(); $m => {%{$_->{status}}} } values %$mods;
+    ### this needs fixink for new Status.pm!!! ###
+    #my %return = map { my $m = $_->module(); $m => {%{$_->{status}}} } values %$mods;
+
+    ### Fix:
+    my %return = map { $_->module => $_->status } values %$mods;
 
     return \%return;
 
@@ -209,12 +230,22 @@ sub _install_module {
 ### containing a list of modules that are part of the bundle in a certain format
 sub _install_bundle {
     my $self = shift;
-    my %args = @_;
+    my %hash = @_;
+    my $err  = $self->error_object;
+    my $conf = $self->configure_object;
 
-    my $err = $self->{_error};
+
+    my $tmpl = {
+        dir     => { required => 1, default => [], strict_type => 1 },
+        verbose => { default => $conf->get_conf('verbose') },
+    };
+
+    my $args = check( $tmpl, \%hash ) or return undef;
+
+    my $verbose = $args->{verbose};
 
     my $flag;
-    for my $dir ( @{$args{'dir'}} ) {
+    for my $dir ( @{$args->{'dir'}} ) {
         my $dh = new DirHandle;
 
         ### open the dir of the newly extracted bundle
@@ -272,13 +303,21 @@ sub _install_bundle {
 ### method to read the names of modules that are mentioned in a bundle
 sub _bundle_files {
     my $self = shift;
-    my %args = @_;
-    my $conf = $self->{_conf};
-    my $err  = $self->{_error};
+    my %hash = @_;
+    my $conf = $self->configure_object;
+    my $err  = $self->error_object;
+
+    my $tmpl = {
+        file    => { required => 1, allow => sub { -e pop() && -s _ }  },
+    };
+
+    my $args = check( $tmpl, \%hash ) or return undef;
+
+    my $verbose = $args->{verbose};
 
     my $fh = new FileHandle;
-    $fh->open($args{file}) or (
-        $err->trap( error => qq[could not open $args{file}: $!] ),
+    $fh->open($args->{file}) or (
+        $err->trap( error => qq[could not open $args->{file}: $!] ),
         return 0
     );
 
@@ -305,7 +344,7 @@ sub _bundle_files {
 
             $err->inform(
                 msg   => loc("Installing bundled module: %1", $name),
-                quiet => !$conf->get_conf('verbose'),
+                quiet => !$verbose,
             );
 
             push @list, $modobj;
@@ -315,6 +354,8 @@ sub _bundle_files {
 }
 
 
+### XXX this one can be replaced by CPANPLUS::Tools::Module::check_install()
+### XXX but it would have to have the localization redone
 
 ### this checks if a certain module is installed already ###
 ### if it returns true, the module in question is already installed
@@ -326,136 +367,68 @@ sub _bundle_files {
 ### a hashref with he module name and version in it.. so 'true' as well.
 sub _check_install {
     my $self = shift;
-    my %args = @_;
+    my %hash = @_;
+    my $err  = $self->error_object;
+    my $conf = $self->configure_object;
 
-    my $conf = $self->{_conf};
-    my $err  = $self->{_error};
+    local $CPANPLUS::Tools::Check::ALLOW_UNKNOWN = 1;
 
-    my $version = defined $args{version} # prevent warnings
-                ? $args{version}
-                : ( $self->_module_tree->{$args{module}}->{version} || '0.0' );
+    my $tmpl = {
+            module  => { required   => 1        },
+            verbose => { default    => $conf->get_conf('verbose') },
+    };
 
-    ### avoid the 'undef' isn't numeric - warning ###
-    $version = '0.0', if $version =~ /^undef/i;
+    my $args = check( $tmpl, \%hash ) or return undef;
 
-    my $module = File::Spec->catfile( split(/::/, $args{module}) );
-
-    my $href = {};
-
-    DIR: for my $dir ( @INC ) {
-        my ($fh, $filename);
-        if ( ref $dir ) {
-            # @INC hook -- we invoke it and get the filehandle back
-            $filename = join('/', split(/::/, $args{module})) . '.pm';
-
-            if (UNIVERSAL::isa($dir, 'CODE')) {
-                ($fh) = $dir->($dir, $filename);
-            }
-            elsif (UNIVERSAL::isa($dir, 'ARRAY')) {
-                ($fh) = $dir->[0]->($dir, $filename, @{$dir}{1..$#{$dir}})
-            }
-            elsif (UNIVERSAL::can($dir, 'INC')) {
-                ($fh) = $dir->INC->($dir, $filename);
-            }
-
-            if (!UNIVERSAL::isa($fh, 'GLOB')) {
-                $err->trap( error => loc("Can't open %1: %2", $filename, $!) );
-                next;
-            }
-        }
-        else {
-            $filename = File::Spec->catfile($dir, "$module.pm");
-            next unless -e $filename;
-
-            $fh = new FileHandle;
-            if (!$fh->open($filename)) {
-                $err->trap( error => loc("Can't open %1: %2", $filename, $!) );
-                next;
-            }
-        }
-
-        ### defaults ###
-        $href = {
-            file        => $filename,
-            uptodate    => 0,
-            version     => 0,
-        };
-
-
-        local $_; # the 'while (<IN>)' has issues -jmb
-        while (<$fh>) {
-
-            # the following regexp comes from the ExtUtils::MakeMaker
-            # documentation.
-            if (/([\$*])(([\w\:\']*)\bVERSION)\b.*\=/) {
-
-                ### this will eval the version in to $VERSION if it
-                ### was declared as $VERSION in the module.
-                ### else the result will be in $res.
-                ### this is a fix on skud's Module::InstalledVersion
-                {   local $VERSION;
-                    #local $@; - can't use this, it's buggy -kane
-                    my $res = eval $_;
-
-                    ### default to '0.0' if there REALLY is no version
-                    ### all to satisfy warnings
-                    #$href->{version} = $VERSION || $res;
-                    $href->{version} = $VERSION || $res || '0.0';
-
-                    $href->{uptodate} = 1 if ($version <= $href->{version});
-                    close $fh;
-                    return $href;
-                }
-            }
-        }
-
-        ### only complain if we expected fo find a version higher than 0.0 anyway ###
-        if( $version > 0 ) {
-            ### if we got here, we didn't find the version
-            $err->inform(
-                msg     => loc("Can't check version on %1... assuming it's up to date", $args{module}),
-                quiet   => !$conf->get_conf('verbose')
-            );
-        }
-
-        $href->{uptodate} = 1;
+    if (my $href = check_install(%hash)) {
         return $href;
+    } else {
+        $self->error_object->trap(
+                    error => loc(qq[Could not find '%1'], $args->{module})
+                ) if $args->{verbose};
+        return undef;
     }
-
-    return $href;
 }
 
 ### uninstall a module ###
 sub _uninstall {
     my $self = shift;
-    my %args = @_;
+    my %hash = @_;
+    my $conf = $self->configure_object;
+    my $err  = $self->error_object;
 
-    my $conf = $self->{_conf};
-    my $err  = $self->{_error};
+    local $CPANPLUS::Tools::Check::ALLOW_UNKNOWN = 1;
+
+    my $tmpl = {
+        module  => { required => 1 },
+        verbose => { default => $conf->get_conf('verbose') },
+    };
+    my $args = check( $tmpl, \%hash ) or return undef;
+
+    my $verbose = $args->{verbose};
+    my $mod     = $args->{module};
 
-    ### all to please warnings...
-    my $mod = $args{'module'};
 
     ### we can pass the arguments straight on to _files()
     my $files;
-    unless ( $files = $self->_files(%args) ) {
+    unless ( $files = $self->_files(%$args) ) {
         $err->trap( error => loc("No files found for %1!", $mod) );
         return;
     }
 
     ### ditto for _directories()
     my $dirtree;
-    unless ( $dirtree = $self->_directories(%args) ) {
+    unless ( $dirtree = $self->_directories(%$args) ) {
         $err->trap( error => loc("No directory tree found for %1!", $mod) );
         return;
     }
 
     ### remove the packlist file altogether
     my $flag;
-    for my $file ( @$files, $self->_packlist_file(%args)) {
+    for my $file ( @$files, $self->_packlist_file(%$args)) {
         $err->inform(
                     msg     => loc("unlinking %1", $file),
-                    quiet   => !$conf->get_conf('verbose')
+                    quiet   => !$verbose
                 );
 
         unless (unlink $file) {
@@ -465,10 +438,6 @@ sub _uninstall {
     }
 
     for my $dir ( sort @$dirtree ) {
-        ### Check if the $dir is empty
-        use Cwd;
-
-        #print "I am now in: ", cwd, "\n";
 
         local *DIR;
         opendir DIR, $dir;
@@ -479,16 +448,16 @@ sub _uninstall {
 
         $err->inform(
                     msg     => loc("removing %1", $dir),
-                    quiet   => !$conf->get_conf('verbose')
+                    quiet   => !$verbose
                 );
 
         unless (rmdir $dir) {
-            $err->trap( error => loc("could not remove %1: %2", $dir, $!) );
 
             ### this fails on my win2k machines.. it indeed leaves the
             ### dir, but it's not a critical error, since the files have
             ### been removed. --kane
-            #$flag = 1;
+            $err->trap( error => loc("could not remove %1: %2", $dir, $!) )
+                unless $^O eq 'MSWin32';
         }
     }
 
@@ -497,12 +466,21 @@ sub _uninstall {
 
 sub _check_md5 {
     my $self = shift;
-    my %args = @_;
+    my %hash = @_;
+    my $conf = $self->configure_object;
+    my $err  = $self->error_object;
 
-    my $conf = $self->{_conf};
-    my $err  = $self->{_error};
+    my $tmpl = {
+        verbose     => { default => $conf->get_conf('verbose') },
+        data        => { allow => sub { ref $_[1] &&
+                                        $_[1]->isa('CPANPLUS::Internals::Module')
+                                  } },
+    };
 
-    my $href = $args{'data'};
+    my $args = check( $tmpl, \%hash ) or return undef;
+
+    my $modobj  = $args->{'data'};
+    my $verbose = $args->{verbose};
 
     ### check prerequisites
     my $use_list = { 'Digest::MD5' => '0.0' };
@@ -512,11 +490,11 @@ sub _check_md5 {
         my $basedir = File::Spec->catdir(
                         $conf->_get_build('base'),
                         $conf->_get_ftp('base'),
-                        $href->{'path'},
+                        $modobj->path,
                     );
 
         ### full path to both the archive and the checksums file ###
-        my $archive = File::Spec->catfile( $basedir, $href->{'package'} );
+        my $archive = File::Spec->catfile( $basedir, $modobj->package );
         my $cs_file = File::Spec->catfile( $basedir, 'CHECKSUMS' );
 
         my $fh = new FileHandle;
@@ -524,7 +502,7 @@ sub _check_md5 {
         ### open the archive file ###
         unless ( $fh->open($archive) ) {
             $err->trap( error => loc("Could not open %1: %2", $archive, $!) );
-            return 0;
+            return undef;
         }
 
         ### set binmode, VERY important on windows ###
@@ -540,25 +518,25 @@ sub _check_md5 {
         my $flag;
         if ( -e $cs_file ) { $flag = 1; }
 
-        my $checksums = $self->_get_checksums( mod => $href );
+        my $checksums = $self->_get_checksums( mod => $modobj );
 
         CHECK: {
-            if ( $checksums->{ $href->{'package'} }->{'md5'} eq $digest ) {
+            if ( $checksums->{ $modobj->package }->{'md5'} eq $digest ) {
                 $err->inform(
                         msg     => loc("Checksum for %1 OK", $archive),
-                        quiet   => !$conf->get_conf('verbose')
+                        quiet   => !$verbose
                 );
                 return 1;
 
             } else {
-                if ( $flag and !(keys %{$checksums->{$href->{'package'}}}) ) {
+                if ( $flag and !(keys %{$checksums->{ $modobj->package }}) ) {
                     ### if we didnt already have a checksums file on our disk, we might have
                     ### an outdated one... in that case we'll refetch it and try again to see
                     ### if we get a matching MD5 -kane
-                    unless( $checksums = $self->_get_checksums( mod => $href, force => 1 ) ) {
+                    unless( $checksums = $self->_get_checksums( mod => $modobj, force => 1 ) ) {
                         $err->trap(
                             error => loc("Unable to fetch checksums file! Can not verify this distribution is safe!") );
-                        return 0;
+                        return undef;
                     }
 
                     $flag = 1;
@@ -566,8 +544,8 @@ sub _check_md5 {
 
                 } else {
 
-                    $err->trap( error => loc("MD5 sums did not add up for %1", $href->{package}) );
-                    return 0;
+                    $err->trap( error => loc("MD5 sums did not add up for %1", $modobj->package) );
+                    return undef;
                 }
             }
         } #CHECK
@@ -575,7 +553,7 @@ sub _check_md5 {
     } else {
 
         $err->trap(
-            error => loc("You don't have %1! Please install it as soon as possible! Assuming %1 is trustworthy", "Digest::MD5", $href->{package})
+            error => loc("You don't have %1! Please install it as soon as possible! Assuming %2 is trustworthy", "Digest::MD5", $modobj->package)
             );
         return 1;
     }
@@ -583,37 +561,86 @@ sub _check_md5 {
 
 sub _check_signature {
     my $self = shift;
-    my %args = @_;
+    my %hash = @_;
+    my $conf = $self->configure_object;
+    my $err  = $self->error_object;
 
-    my $conf = $self->{_conf};
-    my $err  = $self->{_error};
+    my $tmpl = {
+        verbose     => { default => $conf->get_conf('verbose') },
+        dir         => { allow => sub { -d pop() } },
+    };
 
-    my $dir = $args{'dir'};
+    my $args = check( $tmpl, \%hash ) or return undef;
+
+    my $verbose = $args->{verbose};
+    my $dir     = $args->{'dir'};
 
     ### check prerequisites
     my $use_list = { 'Module::Signature' => '0.06' };
 
     if ($self->_can_use(modules => $use_list)) {
-        use Cwd;
         my $old_cwd = Cwd::cwd();
 
-        chdir $dir;
+        return undef unless $self->_chdir( dir => $dir );
+
         my $rv = Module::Signature::verify();
 
         if ($rv eq Module::Signature::SIGNATURE_OK() or
             $rv eq Module::Signature::SIGNATURE_MISSING()) {
+
+            $self->_chdir( dir => $old_cwd );
             return 1;
         }
 
-        return 0;
+    } else {
+        $err->trap(
+            error => loc("You do not have %1! Please install it as soon as possible! ".
+                         "Assuming %2 is trustworthy", "Module::Signature", $dir)
+        );
+        return 1;
+    }
+
+    return undef;
+}
+
+
+sub _check_detached_signature {
+    my $self = shift;
+    my %hash = @_;
+    my $conf = $self->configure_object;
+    my $err  = $self->error_object;
+
+    my $tmpl = {
+        mod         => { required => 1 },
+        verbose     => { default => $conf->get_conf('verbose') },
+    };
+
+    my $args = check( $tmpl, \%hash ) or return undef;
+    my $modobj = $args->{mod};
+
+    # XXX - more robust checking needed
+    eval {
+        $self->_get_checksums( mod => $modobj )->{ $modobj->package . ".sig"}
+    } or return 1;
+
+    my $local_file = $self->_get_detached_signature( mod => $modobj )
+        or return undef;
+
+    ### check prerequisites
+    my $use_list = { 'Module::Signature' => '0.26' };
+
+    if ($self->_can_use(modules => $use_list)) {
+        return Module::Signature::_verify($local_file) == Module::Signature::SIGNATURE_OK();
     }
     else {
         $err->trap(
-            error => loc("You don't have %1! Please install it as soon as possible! Assuming %1 is trustworthy", "Module::Signature", $dir)
+            error => loc("You do not have %1! Please install it as soon as possible! ".
+                         "Assuming %2 is trustworthy", "Module::Signature", $modobj->package)
         );
         return 1;
     }
 }
+
 1;
 
 # Local variables:
